#!/usr/bin/env perl

# outline.pl
#   quick and dirty code by Matthew McCowan


# $state
#   0   - waiting on the title
#   1   - we have a multiline title
#   2   - into the body of the work

use strict;
use warnings;
$|++;

my ($line
   ,$front
   ,$back
   ,$state
   ,$title
   ,@titles
   ,@body
   ,$indent_level
   );

@titles  = ();
@body    = ();
$state   = 0;

while(<>) {
    chomp();
    $line  = $_;
    $line =~ s/\s+$//;
    
    if ($state == 0) {
        if ($line =~ m/^\s*(\=|\#+)/) {
            $state  = 2;      # skipped a title - legal per spec
            $line =~ s/^\s+//;
            push(@body, $line);
        } elsif ($line =~ m/^Title\:/i) {
            if ($line =~ m/^Title\:$/i) {
                $state  = 1;
            } else {
                $line =~ s/^Title\:\s+//i;
                push(@titles, $line);
                $state  = 2;
            }
        }
    } elsif ($state == 1) {
        if ($line =~ m/^\s*$/ || $line =~ m/^\s*[A-Za-z0-9\_\-]+\:/) {
            $state  = 2;
        } else {
            $line =~ s/^\s+//;
            push(@titles, $line);
        }
    } else { # $state == 2
        $line =~ s/^\s+//;
        push(@body, $line) if $line =~ m/^(\=|\#+)/;
    }
}

foreach $title (@titles) {
    $title =~ s/^\s*[\_\*]*//;
    $title =~ s/[\_\*]*\s*$//;
    print "\# $title\n";
}
print "\n" if scalar(@titles) > 0;

$indent_level  = 0;
foreach $line (@body) {
    if ($line =~ m/^\#+/) {
        $front         = $&;
        $back          = $';
        $indent_level  = length($front);
        $back         =~ s/^\s+//;
        
        print "\t" x ($indent_level - 1);
        print "- $back\n\n";
    } else {
        $line =~ s/^\=\s*//;
        print "\t" x $indent_level;
        print "$line\n\n";
    }
}